@@ -98,9 +98,35 @@ sub get_all_method_names {
 
 sub add_attribute {
     my $self = shift;
-    my $attr = shift;
 
-    $self->{'attributes'}{$attr->name} = $attr;
+    if (@_ == 1 && blessed($_[0])) {
+        my $attr = shift @_;
+        $self->{'attributes'}{$attr->name} = $attr;
+    } else {
+        my $names = shift @_;
+        $names = [$names] if !ref($names);
+        my $metaclass = 'Mouse::Meta::Attribute';
+        my %options = @_;
+
+        if ( my $metaclass_name = delete $options{metaclass} ) {
+            my $new_class = Mouse::Util::resolve_metaclass_alias(
+                'Attribute',
+                $metaclass_name
+            );
+            if ( $metaclass ne $new_class ) {
+                $metaclass = $new_class;
+            }
+        }
+
+        for my $name (@$names) {
+            if ($name =~ s/^\+//) {
+                $metaclass->clone_parent($self, $name, @_);
+            }
+            else {
+                $metaclass->create($self, $name, @_);
+            }
+        }
+    }
 }
 
 sub compute_all_applicable_attributes {
@@ -187,25 +213,36 @@ sub attribute_metaclass { "Mouse::Meta::Class" }
 
 sub _install_modifier {
     my ( $self, $into, $type, $name, $code ) = @_;
-    if (eval "require Class::Method::Modifiers::Fast; 1") {
-        Class::Method::Modifiers::Fast::_install_modifier( 
-            $into,
-            $type,
-            $name,
-            $code
-        );
-    }
-    elsif (eval "require Class::Method::Modifiers; 1") {
-        Class::Method::Modifiers::_install_modifier( 
-            $into,
-            $type,
-            $name,
-            $code
-        );
-    }
-    else {
-        Carp::croak("Method modifiers require the use of Class::Method::Modifiers. Please install it from CPAN and file a bug report with this application.");
+
+    # which is modifer class available?
+    my $modifier_class = do {
+        if (eval "require Class::Method::Modifiers::Fast; 1") {
+            'Class::Method::Modifiers::Fast';
+        } elsif (eval "require Class::Method::Modifiers; 1") {
+            'Class::Method::Modifiers';
+        } else {
+            Carp::croak("Method modifiers require the use of Class::Method::Modifiers or Class::Method::Modifiers::Fast. Please install it from CPAN and file a bug report with this application.");
+        }
+    };
+    my $modifier = $modifier_class->can('_install_modifier');
+
+    # replace this method itself :)
+    {
+        no strict 'refs';
+        no warnings 'redefine';
+        *{__PACKAGE__ . '::_install_modifier'} = sub {
+            my ( $self, $into, $type, $name, $code ) = @_;
+            $modifier->(
+                $into,
+                $type,
+                $name,
+                $code
+            );
+        };
     }
+
+    # call me. for first time.
+    $self->_install_modifier( $into, $type, $name, $code );
 }
 
 sub add_before_method_modifier {
@@ -340,7 +377,7 @@ Returns the name of the owner class.
 
 Gets (or sets) the list of superclasses of the owner class.
 
-=head2 add_attribute Mouse::Meta::Attribute
+=head2 add_attribute (Mouse::Meta::Attribute| name => spec)
 
 Begins keeping track of the existing L<Mouse::Meta::Attribute> for the owner
 class.
